@@ -16,13 +16,16 @@
 # under the License.
 
 import os
+
 from testscenarios.testcase import TestWithScenarios
 from testtools import TestCase
+
 from jenkins_jobs.modules import publishers
-from tests.base import get_scenarios, BaseTestCase
+from tests.base import BaseTestCase
+from tests.base import get_scenarios
 
 
-class TestCaseModulePublishers(TestWithScenarios, TestCase, BaseTestCase):
+class TestCaseModulePublishers(TestWithScenarios, BaseTestCase, TestCase):
     fixtures_path = os.path.join(os.path.dirname(__file__), 'fixtures')
     scenarios = get_scenarios(fixtures_path)
     klass = publishers.Publishers
